from typing import List, Tuple, Union, overload


ERR_OK: int
ERR_OPEN: int
ERR_NOFS: int
ERR_REGMSG: int
ERR_ATOM: int
ERR_MAP: int
ERR_VIEW: int
ERR_VERSION: int
ERR_WRONGFS: int
ERR_NOTOPEN: int
ERR_NODATA: int
ERR_TIMEOUT: int
ERR_SENDMSG: int
ERR_DATA: int
ERR_RUNNING: int
ERR_SIZE: int
SIM_ANY: int
SIM_FS98: int
SIM_FS2K: int
SIM_CFS2: int
SIM_CFS1: int
SIM_FLY: int
SIM_FS2K2: int
SIM_FS2K4: int
SIM_FSX: int
SIM_ESP: int
SIM_P3D: int
SIM_FSX64: int
SIM_P3D64: int


class FSUIPCException(Exception):
    def __init__(self, code: int) -> None:
        ...


class OpaqueData:
    ...


def open(version: int) -> None:
    ...


def close() -> None:
    ...


def prepare_data(
    data_specification: List[Tuple[int, Union[int, str]]], for_reading: bool
) -> OpaqueData:
    ...


def read(
    prepared_or_unprepared_data: Union[OpaqueData, List[Tuple[int, Union[int, str]]]]
) -> List[Union[int, float, bytes]]:
    ...


@overload
def write(data: List[Tuple[int, Union[int, str], Union[int, float, bytes]]]) -> None:
    ...


@overload
def write(prepared_data: OpaqueData, data: List[Union[int, float, bytes]]) -> None:
    ...



from typing import List, Tuple, Union
import pyuipc


DataSpecification = List[Tuple[int, Union[int, str]]]


class PreparedData():
    """Represents a data specification for reading and writing data.

    Use this class when repeatedly reading or writing the same set of offsets
    to reduce the cost of converting the sets of offsets and their types.

    Instances of PreparedData should be created using the FSUIPC class's
    prepare_data method.
    """

    def __init__(self, data_specification: DataSpecification, for_reading: bool) -> None:
        self._prepared_data = pyuipc.prepare_data(data_specification, for_reading)

    def read(self) -> List[Union[int, float, bytes]]:
        """Read data from FSUIPC.

        Returns a list of the data items read.
        """

        return pyuipc.read(self._prepared_data)

    def write(self, data: List[Union[int, float, bytes]]) -> None:
        """Write data to FSUIPC.

        Arguments:
        data - a list of values to write to the predefined offsets
        """

        pyuipc.write(self._prepared_data, data)



import platform
import setuptools
import struct
import sys


with open("README.md", "r") as fh:
    long_description = fh.read()


if struct.calcsize("P") != 4 or platform.system() != "Windows":
    sys.stderr.write("This package requires a 32-bit version of Python for Windows\n")
    exit(1)


setuptools.setup(
    name="fsuipc",
    version="1.0.3",
    author="Tim Jensen",
    author_email="tim.l.jensen@gmail.com",
    description="Client wrapper for FSUIPC",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/tjensen/fsuipc",
    project_urls={
        "Source": "https://github.com/tjensen/fsuipc",
        "Tracker": "https://github.com/tjensen/fsuipc/issues"
    },
    keywords=["flight", "simulator", "pyuipc"],
    packages=["fsuipc", "pyuipc-stubs"],
    package_data={
        "fsuipc": ["py.typed"],  # PEP 561
        "pyuipc-stubs": ["__init__.pyi"]
    },
    zip_safe=False,  # Enables mypy to find the installed package
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Win32 (MS Windows)",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Games/Entertainment :: Simulation"
    ],
    python_requires=">=3.5,<3.9",
    setup_requires=["wheel"],
    ext_modules=[setuptools.Extension(
        name="pyuipc",
        sources=[
            "src/UIPC_SDK_C/IPCuser.c",
            "src/UIPC_SDK_Python/pyuipc.cc"
        ],
        include_dirs=["src/UIPC_SDK_C"],
        libraries=["user32"])]
)



from types import TracebackType
from typing import List, Optional, Tuple, Type, Union

from fsuipc.prepared_data import PreparedData, DataSpecification
import pyuipc


class FSUIPC():
    """A connection to FSUIPC.

    Note that FSUIPC objects can be used as context managers so that the
    connection is automatically closed upon exiting the context. For
    example:

        with fsuipc.FSUIPC() as ipc:
            altitude = ipc.read([(0x570, "l")])
    """

    def __init__(self, version: int = pyuipc.SIM_ANY) -> None:
        """Opens a connection to FSUIPC.

        By default, connects to any available flight simulator.

        Arguments:
        version -- the type of flight simulator to connect to (e.g. fsuipc.SIM_FSX)
        """

        pyuipc.open(version)

    def close(self) -> None:
        """Closes the connection to FSUIPC."""

        pyuipc.close()

    def prepare_data(
            self, data_specification: DataSpecification, for_reading: bool
    ) -> PreparedData:
        """Prepare a data specification for reading or writing.

        Preparing a data specification can reduce the cost of converting sets
        of offsets and their types when repeatedly reading or writing the
        same sets of offsets.

        Returns a PreparedData object, which can be used to read and write the
        data.

        Arguments:
        data_specification -- a list of tuples of two items, as described below
        for_reading -- if True, prepare for reading and writing; False for writing, only

        Data Specification:
        The first item of each tuple should be an offset to read or write.

        The second item denotes the type of the value to be read from or
        written into the offset. It should be either a string or an integer.
        If it is a string, it denotes a type of a fixed size:

            - b: a 1-byte unsigned value, to be converted into a Python int
            - c: a 1-byte signed value, to be converted into a Python int
            - h: a 2-byte signed value, to be converted into a Python int
            - H: a 2-byte unsigned value, to be converted into a Python int
            - d: a 4-byte signed value, to be converted into a Python int
            - u: a 4-byte unsigned value, to be converted into a Python long
            - l: an 8-byte signed value, to be converted into a Python long
            - L: an 8-byte unsigned value, to be converted into a Python long
            - f: an 8-byte floating point value, to be converted into a Python double
            - F: a 4-byte floating point value, to be converted into a Python double

        If the second item is an integer, it denotes a string. If the value
        is positive, the string is of a fixed length with no zero terminator.
        If the value is negative, its absolute value is the maximal length of
        the string, but the string itself is zero terminated. Such data is
        returned as a Python bytes object when reading, and a bytes object is
        expected to be written.
        """

        return PreparedData(data_specification, for_reading)

    def read(self, unprepared_data: DataSpecification) -> List[Union[int, float, bytes]]:
        """Read data from FSUIPC using an unprepared data specification.

        Returns a list of the data items read.

        Arguments:
        unprepared_data - a list of tuples of two items, as described by prepare_data
        """

        return pyuipc.read(unprepared_data)

    def write(
            self, unprepared_data: List[Tuple[int, Union[int, str], Union[int, float, bytes]]]
    ) -> None:
        """Write data to FSUIPC using an unprepared data specification.

        Arguments:
        unprepared_data - a list of tuples of three items, as described below

        Data Specification:
        The first two items of each tuple are the offset and data type, as
        described by prepare_data. The third item of each tuple is the actual
        data to write at the given offset.
        """

        return pyuipc.write(unprepared_data)

    def __enter__(self) -> "FSUIPC":
        return self

    def __exit__(
            self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException],
            traceback: Optional[TracebackType]
    ) -> None:
        self.close()
